@@ -37,10 +37,11 @@ namespace {
 enum {
     INDEX_LOADING_VIEW = 0,
     INDEX_TABLE_VIEW,
-    INDEX_LOADING_FAILED_VIEW
+    INDEX_LOADING_FAILED_VIEW,
+    INDEX_EMPTY_VIEW
 };
 
-const char *kLoadingFaieldLabelName = "loadingFailedText";
+const char *kLoadingFailedLabelName = "LoadingFailedText";
 const int kToolBarIconSize = 20;
 const int kStatusBarIconSize = 24;
 //const int kStatusCodePasswordNeeded = 400;
@@ -106,6 +107,7 @@ FileBrowserDialog::FileBrowserDialog(const Account &account, const ServerRepo& r
     createToolBar();
     createStatusBar();
     createLoadingFailedView();
+    createEmptyView();
     createFileTable();
 
     QWidget* widget = new QWidget;
@@ -125,6 +127,7 @@ FileBrowserDialog::FileBrowserDialog(const Account &account, const ServerRepo& r
     stack_->insertWidget(INDEX_LOADING_VIEW, loading_view_);
     stack_->insertWidget(INDEX_TABLE_VIEW, table_view_);
     stack_->insertWidget(INDEX_LOADING_FAILED_VIEW, loading_failed_view_);
+    stack_->insertWidget(INDEX_EMPTY_VIEW, empty_view_);
     stack_->setContentsMargins(0, 0, 0, 0);
 
     vlayout->addWidget(toolbar_);
@@ -282,7 +285,7 @@ void FileBrowserDialog::createStatusBar()
     connect(upload_file_action_, SIGNAL(triggered()), this, SLOT(chooseFileToUpload()));
     upload_menu_->addAction(upload_file_action_);
 
-    // Submenu's Action 2: Upload File (only pro version's server supports it)
+    // Submenu's Action 2: Upload directory (only pro version's server supports it)
     if (account_.isPro()) {
         upload_directory_action_ = new QAction(tr("Upload a directory"), upload_menu_);
         connect(upload_directory_action_, SIGNAL(triggered()), this, SLOT(chooseDirectoryToUpload()));
@@ -384,7 +387,7 @@ void FileBrowserDialog::onGetDirentsFailed(const ApiError& error)
 
 void FileBrowserDialog::onMkdirButtonClicked()
 {
-    QString name = QInputDialog::getText(this, tr("Create a folder"),
+    QString name = seafApplet->getText(this, tr("Create a folder"),
         tr("Folder name"));
     name = name.trimmed();
 
@@ -408,23 +411,23 @@ void FileBrowserDialog::onMkdirButtonClicked()
 
 void FileBrowserDialog::createLoadingFailedView()
 {
-    loading_failed_view_ = new QWidget(this);
-
-    QVBoxLayout *layout = new QVBoxLayout;
-    loading_failed_view_->setLayout(layout);
-
-    QLabel *label = new QLabel;
-    label->setObjectName(kLoadingFaieldLabelName);
+    loading_failed_view_ = new QLabel;
+    loading_failed_view_->setObjectName(kLoadingFailedLabelName);
     QString link = QString("<a style=\"color:#777\" href=\"#\">%1</a>").arg(tr("retry"));
     QString label_text = tr("Failed to get files information<br/>"
                             "Please %1").arg(link);
-    label->setText(label_text);
-    label->setAlignment(Qt::AlignCenter);
+    loading_failed_view_->setText(label_text);
+    loading_failed_view_->setAlignment(Qt::AlignCenter);
 
-    connect(label, SIGNAL(linkActivated(const QString&)),
+    connect(loading_failed_view_, SIGNAL(linkActivated(const QString&)),
             this, SLOT(forceRefresh()));
+}
 
-    layout->addWidget(label);
+void FileBrowserDialog::createEmptyView()
+{
+    empty_view_ = new QLabel;
+    empty_view_->setText(tr("This folder is empty."));
+    empty_view_->setAlignment(Qt::AlignCenter);
 }
 
 void FileBrowserDialog::onDirentClicked(const SeafDirent& dirent)
@@ -640,9 +643,9 @@ void FileBrowserDialog::uploadMultipleFile(const QStringList& names,
     QStringList fnames;
     Q_FOREACH(const QString &name, names) {
         const QFileInfo file = name;
-        // only files are allowed
-        if (!file.isDir() && file.path() == path
-            /* && !account_.isPro*/) {
+        if (file.isDir()) {
+            uploadOrUpdateFile(name);
+        } else if (file.path() == path) {
             fnames.push_back(file.fileName());
         }
     }
@@ -835,14 +838,20 @@ bool FileBrowserDialog::setPasswordAndRetry(FileNetworkTask *task)
 
 void FileBrowserDialog::goForward()
 {
-    QString path = forward_history_.pop();
+    QString path;
+    if (!forward_history_.empty()) {
+        path = forward_history_.pop();
+    }
     backward_history_.push(current_path_);
     enterPath(path);
 }
 
 void FileBrowserDialog::goBackward()
 {
-    QString path = backward_history_.pop();
+    QString path;
+    if (!backward_history_.empty()) {
+        path = backward_history_.pop();
+    }
     forward_history_.push(current_path_);
     enterPath(path);
 }
@@ -853,13 +862,24 @@ void FileBrowserDialog::goHome()
         return;
     }
     backward_history_.push(current_path_);
+    forward_history_.clear();
     enterPath("/");
 }
 
 void FileBrowserDialog::updateTable(const QList<SeafDirent>& dirents)
 {
+    // Commented out because the empty view can't handle file drag & drop events.
+
+    // if (dirents.isEmpty()) {
+    //     stack_->setCurrentIndex(INDEX_EMPTY_VIEW);
+    // } else {
+    //     table_model_->setDirents(dirents);
+    //     stack_->setCurrentIndex(INDEX_TABLE_VIEW);
+    // }
+
     table_model_->setDirents(dirents);
     stack_->setCurrentIndex(INDEX_TABLE_VIEW);
+
     if (!forward_history_.empty()) {
         forward_button_->setEnabled(true);
     } else {
@@ -899,6 +919,8 @@ void FileBrowserDialog::onNavigatorClick(int id)
     for(int i = 1; i <= id; i++)
       path += current_lpath_[i] + "/";
 
+    backward_history_.push(current_path_);
+    forward_history_.clear();
     enterPath(path);
 }
 
@@ -911,7 +933,7 @@ void FileBrowserDialog::onGetDirentRename(const SeafDirent& dirent,
                                           QString new_name)
 {
     if (new_name.isEmpty()) {
-        new_name = QInputDialog::getText(this, tr("Rename"),
+        new_name = seafApplet->getText(this, tr("Rename"),
                    QObject::tr("Rename %1 to").arg(dirent.name),
                    QLineEdit::Normal,
                    dirent.name);
