@@ -823,7 +823,11 @@ void FileTableView::dropEvent(QDropEvent *event)
 
     event->accept();
 
-    emit dropFile(paths);
+    if (parent_->current_readonly_) {
+        seafApplet->warningBox(tr("You do not have permission to upload to this folder"));
+    } else {
+        emit dropFile(paths);
+    }
 }
 
 void FileTableView::dragMoveEvent(QDragMoveEvent *event)
@@ -1099,14 +1103,17 @@ bool FileTableSortFilterProxyModel::lessThan(const QModelIndex &left, const QMod
 {
     bool is_dir_left = source_model_->direntAt(left.row())->isDir();
     bool is_dir_right = source_model_->direntAt(right.row())->isDir();
-    if (is_dir_left != is_dir_right)
+    if (is_dir_left != is_dir_right) {
         return sortOrder() != Qt::AscendingOrder ? is_dir_right
                                                  : !is_dir_right;
-    else {
+    }
+    else if ((left.column() == FILE_COLUMN_NAME) &&
+             (right.column() == FILE_COLUMN_NAME)) {
         const QString left_name = source_model_->direntAt(left.row())->name;
         const QString right_name = source_model_->direntAt(right.row())->name;
-        return digitalCompare(left_name, right_name) < 0 ? true : false;
+        return digitalCompare(left_name, right_name) < 0;
+    }
+    else {
+        return QSortFilterProxyModel::lessThan(left, right);
     }
-
-    return QSortFilterProxyModel::lessThan(left, right);
 }
