@@ -272,6 +272,7 @@ void FileUploadDirectoryTask::createFileServerTask(const QString& link)
     if (local_path_ == "/")
         qWarning("attempt to upload the root directory, you should avoid it\n");
     QDir dir(local_path_);
+
     QDirIterator iterator(dir.absolutePath(), QDirIterator::Subdirectories);
     // XXX (lins05): Move these operations into a thread
     while (iterator.hasNext()) {
@@ -296,13 +297,19 @@ void FileUploadDirectoryTask::createFileServerTask(const QString& link)
         }
     }
 
+    if (names.isEmpty() && empty_subfolders_.isEmpty()) {
+        // The folder dragged into cloud file browser is an empty one. We use
+        // the special name "." to represent it.
+        empty_subfolders_.append(".");
+    }
+
     // printf("total empty folders: %d for %s\n", empty_subfolders_.length(), dir.absolutePath().toUtf8().data());
     fileserver_task_ = new PostFilesTask(link, path_, dir.absolutePath(), names, true);
 }
 
 void FileUploadDirectoryTask::onFinished(bool success)
 {
-    if (!success || empty_subfolders_.empty()) {
+    if (!success || (empty_subfolders_.empty())) {
         FileUploadTask::onFinished(success);
         return;
     }
@@ -318,8 +325,17 @@ void FileUploadDirectoryTask::nextEmptyFolder()
     }
 
     QString folder = empty_subfolders_.takeFirst();
+
+    bool create_parents = true;
+    if (folder == ".") {
+        // This is the case of an empty top-level folder.
+        create_parents = false;
+        folder = ::getBaseName(local_path_);
+    }
+
     create_dir_req_.reset(new CreateDirectoryRequest(
-                              account_, repo_id_, ::pathJoin(path_, folder), true));
+                                account_, repo_id_, ::pathJoin(path_, folder), create_parents));
+
     connect(create_dir_req_.data(), SIGNAL(success()),
             this, SLOT(nextEmptyFolder()));
     connect(create_dir_req_.data(), SIGNAL(failed(const ApiError&)),
@@ -688,14 +704,14 @@ void PostFilesTask::prepare()
     transferred_bytes_ = 0;
     total_bytes_ = 0;
 
-    file_sizes.reserve(names_.size());
+    file_sizes_.reserve(names_.size());
     Q_FOREACH(const QString &name, names_)
     {
         QString local_path = ::pathJoin(local_path_, name);
         // approximate the bytes used by http protocol (e.g. the bytes of
         // header)
         qint64 file_size = QFileInfo(local_path).size() + 1024;
-        file_sizes.push_back(file_size);
+        file_sizes_.push_back(file_size);
         total_bytes_ += file_size;
     }
 }
@@ -740,7 +756,7 @@ void PostFilesTask::onPostTaskFinished(bool success)
         return;
     }
 
-    transferred_bytes_ += file_sizes[current_num_];
+    transferred_bytes_ += file_sizes_[current_num_];
     startNext();
 }
 
