@@ -295,6 +295,16 @@ void AccountView::getRepoTokenWhenRelogin(const Account& account)
     if (!new_account.isValid())
         return;
 
+    // For debugging lots of repos problem.
+    // TODO: Comment this out before committing!!
+    //
+    // int targetNumberForDebug = 300;
+    // while (repo_ids.size() < targetNumberForDebug) {
+    //     repo_ids.append(repo_ids);
+    // }
+    // repo_ids = repo_ids.mid(0, 300);
+    // printf ("repo_ids.size() = %d\n", repo_ids.size());
+
     GetRepoTokensRequest *req = new GetRepoTokensRequest(
         new_account, repo_ids);
 
@@ -315,16 +325,8 @@ void AccountView::updateAvatar()
     }
 
     AvatarService *service = AvatarService::instance();
-
-    if (service->avatarFileExists(account.username)) {
-        QString icon_path = AvatarService::instance()->getAvatarFilePath(account.username);
-        mAccountBtn->setIcon(QIcon(icon_path));
-        return;
-    }
-
-    mAccountBtn->setIcon(QIcon(":/images/account.png"));
-    // will trigger a GetAvatarRequest
-    service->getAvatar(account.username);
+    QIcon avatar = QPixmap::fromImage(service->getAvatar(account.username));
+    mAccountBtn->setIcon(QIcon(avatar));
 }
 
 bool AccountView::eventFilter(QObject *obj, QEvent *event)
@@ -395,7 +397,7 @@ void AccountView::toggleAccount()
     connect(req, SIGNAL(success()),
             this, SLOT(onLogoutDeviceRequestSuccess()));
     connect(req, SIGNAL(failed(const ApiError&)),
-            this, SLOT(onLogoutDeviceRequestFailed(const ApiError&)));
+            this, SLOT(onLogoutDeviceRequestSuccess()));
     req->send();
 }
 
@@ -438,19 +440,6 @@ void AccountView::onLogoutDeviceRequestSuccess()
     req->deleteLater();
 }
 
-void AccountView::onLogoutDeviceRequestFailed(const ApiError& error)
-{
-    LogoutDeviceRequest *req = (LogoutDeviceRequest *)QObject::sender();
-    req->deleteLater();
-    QString msg;
-    if (error.httpErrorCode() == 404) {
-        msg = tr("Logging out is not supported on your server (version too low).");
-    } else {
-        msg = tr("Failed to remove information on server: %1").arg(error.toString());
-    }
-    seafApplet->warningBox(msg, this);
-}
-
 void AccountView::onGetRepoTokensSuccess()
 {
     GetRepoTokensRequest *req = (GetRepoTokensRequest *)(sender());
