@@ -2,12 +2,14 @@
 #define SEAFILE_CLIENT_LOADING_VIEW_H_
 
 #include <QWidget>
+#include <QToolButton>
+#include <QLabel>
 
 class QMovie;
 class QShowEvent;
 class QHideEvent;
 
-class LoadingView : public QWidget {
+class LoadingView : public QLabel {
     Q_OBJECT
 public:
     LoadingView(QWidget *parent=0);
@@ -23,4 +25,10 @@ private:
     QMovie *gif_;
 };
 
+class LoadMoreButton : public QToolButton {
+    Q_OBJECT
+public:
+    explicit LoadMoreButton(QWidget *parent=0);
+};
+
 #endif // SEAFILE_CLIENT_LOADING_VIEW_H_
