@@ -36,6 +36,7 @@
 #include "filebrowser/progress-dialog.h"
 #include "ui/set-repo-password-dialog.h"
 #include "ui/private-share-dialog.h"
+#include "ui/check-repo-root-perm-dialog.h"
 
 #include "repo-tree-view.h"
 
@@ -118,6 +119,11 @@ RepoTreeView::RepoTreeView(QWidget *parent)
 
     setAcceptDrops(true);
     setDefaultDropAction(Qt::CopyAction);
+    // setAlternatingRowColors(true);
+    setDropIndicatorShown(true);
+
+    current_drop_target_ = QModelIndex();
+    previous_drop_target_ = QModelIndex();
 }
 
 void RepoTreeView::loadExpandedCategries()
@@ -805,15 +811,43 @@ void RepoTreeView::dropEvent(QDropEvent *event)
 
     RepoItem *item = static_cast<RepoItem*>(standard_item);
     const ServerRepo &repo = item->repo();
-    const QUrl url = event->mimeData()->urls().at(0);
 
+    updateDropTarget(QModelIndex());
+
+    const QUrl url = event->mimeData()->urls().at(0);
     QString local_path = url.toLocalFile();
 #if defined(Q_OS_MAC) && (QT_VERSION <= QT_VERSION_CHECK(5, 4, 0))
         local_path = utils::mac::fix_file_id_url(local_path);
 #endif
+
+    if (repo.readonly) {
+        // Do not call the `show` method of the dialog. It would show itself if
+        // the task doens't finish within 4 seconds.
+        //
+        // This is also why we can't create the dialog object on stack and use
+        // `dialog.exec()`.
+        CheckRepoRootDirPermDialog *dialog = new CheckRepoRootDirPermDialog(
+            seafApplet->accountManager()->currentAccount(), repo, local_path, this);
+        dialog->setAttribute(Qt::WA_DeleteOnClose);
+        connect(dialog, SIGNAL(finished(int)), this, SLOT(checkRootPermDone()));
+    } else {
+        uploadDroppedFile(repo, local_path);
+    }
+}
+
+void RepoTreeView::checkRootPermDone()
+{
+    CheckRepoRootDirPermDialog *dialog = qobject_cast<CheckRepoRootDirPermDialog *>(sender());
+    if (dialog->hasWritePerm()) {
+        uploadDroppedFile(dialog->repo(), dialog->localPath());
+    } else {
+        seafApplet->warningBox(tr("You do not have permission to upload to this folder"));
+    }
+}
+
+void RepoTreeView::uploadDroppedFile(const ServerRepo& repo, const QString& local_path)
+{
     const QString file_name = QFileInfo(local_path).fileName();
-
-
     // if the repo is synced
     LocalRepo local_repo;
     if (seafApplet->rpcClient()->getLocalRepo(repo.id, &local_repo) >= 0) {
@@ -844,22 +878,35 @@ void RepoTreeView::dropEvent(QDropEvent *event)
 
 void RepoTreeView::dragMoveEvent(QDragMoveEvent *event)
 {
-    DropIndicatorPosition position = dropIndicatorPosition();
-    if (position == QAbstractItemView::OnItem) {
-        event->setDropAction(Qt::CopyAction);
-        event->accept();
-        //TODO highlight the selected item, and dehightlight when it's over
-        // const QModelIndex index = indexAt(event->pos());
-        // RepoItem *item = static_cast<RepoItem*>(getRepoItem(index));
-        // if (!item || item->type() != REPO_ITEM_TYPE) {
-        //     return;
-        // }
+    QPoint pos = event->pos();
+    const QModelIndex index = indexAt(pos);
+    QRect rect = visualRect(index);
+
+    // highlight the selected item, and dehightlight when it's over
+    QStandardItem *item = getRepoItem(index);
+    if (item && item->type() == REPO_ITEM_TYPE) {
+        if (changeGrayBackground(pos, rect)) {
+            updateDropTarget(index);
+            event->setDropAction(Qt::CopyAction);
+            event->accept();
+        } else {
+            updateDropTarget(QModelIndex());
+            event->setDropAction(Qt::IgnoreAction);
+            event->accept();
+        }
     } else {
-        event->setDropAction(Qt::IgnoreAction);
-        event->accept();
+       event->setDropAction(Qt::IgnoreAction);
+       event->accept();
+       return;
     }
 }
 
+void RepoTreeView::dragLeaveEvent(QDragLeaveEvent *event)
+{
+    updateDropTarget(QModelIndex());
+    QTreeView::dragLeaveEvent(event);
+}
+
 void RepoTreeView::dragEnterEvent(QDragEnterEvent *event)
 {
     if (event->mimeData()->hasUrls() && event->mimeData()->urls().size() == 1) {
@@ -879,6 +926,44 @@ void RepoTreeView::dragEnterEvent(QDragEnterEvent *event)
     }
 }
 
+bool RepoTreeView::changeGrayBackground(
+    const QPoint& pos, const QRect& rect) const
+{
+    const int margin = 2;
+    if ((pos.y() - rect.top() > margin) &&
+        (rect.bottom() - pos.y() > margin)) {
+        return true;
+    } else {
+        return false;
+    }
+}
+
+void RepoTreeView::updateBackground()
+{
+    if (current_drop_target_.isValid()) {
+        dataChanged(current_drop_target_, current_drop_target_,
+                    QVector<int>(1, Qt::BackgroundRole));
+    }
+
+    if (previous_drop_target_.isValid()) {
+        dataChanged(previous_drop_target_, previous_drop_target_,
+                    QVector<int>(1, Qt::BackgroundRole));
+    }
+}
+
+void RepoTreeView::updateDropTarget(const QModelIndex& index)
+{
+    previous_drop_target_ = current_drop_target_;
+    if (index.isValid() && index == current_drop_target_) {
+        // No need to repaint since the cursor is still with in the same repo
+        // item.
+        return;
+    }
+
+    current_drop_target_ = index;
+    updateBackground();
+}
+
 void RepoTreeView::uploadFileStart(FileUploadTask *task)
 {
     connect(task, SIGNAL(finished(bool)),
@@ -952,6 +1037,7 @@ void RepoTreeView::setRepoSyncInterval()
     dialog.setWindowTitle(
         tr("Set Sync Internval For Library \"%1\"").arg(local_repo.name));
     dialog.setWindowIcon(QIcon(":/images/seafile.png"));
+    dialog.setWindowFlags(dialog.windowFlags() & ~Qt::WindowContextHelpButtonHint);
     dialog.resize(400, 100);
     if (dialog.exec() != QDialog::Accepted) {
         return;
